# Copyright 2020 The SqueezeBert authors and The HuggingFace Inc. team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""SqueezeBERT model configuration"""

from huggingface_hub.dataclasses import strict

from ...configuration_utils import PreTrainedConfig
from ...utils import auto_docstring


@auto_docstring(checkpoint="squeezebert/squeezebert-uncased")
@strict
class SqueezeBertConfig(PreTrainedConfig):
    r"""
    q_groups (`int`, *optional*, defaults to 4):
        The number of groups in Q layer.
    k_groups (`int`, *optional*, defaults to 4):
        The number of groups in K layer.
    v_groups (`int`, *optional*, defaults to 4):
        The number of groups in V layer.
    post_attention_groups (`int`, *optional*, defaults to 1):
        The number of groups in the first feed forward network layer.
    intermediate_groups (`int`, *optional*, defaults to 4):
        The number of groups in the second feed forward network layer.
    output_groups (`int`, *optional*, defaults to 4):
        The number of groups in the third feed forward network layer.

    Examples:

    ```python
    >>> from transformers import SqueezeBertConfig, SqueezeBertModel

    >>> # Initializing a SqueezeBERT configuration
    >>> configuration = SqueezeBertConfig()

    >>> # Initializing a model (with random weights) from the configuration above
    >>> model = SqueezeBertModel(configuration)

    >>> # Accessing the model configuration
    >>> configuration = model.config
    ```
    """

    model_type = "squeezebert"

    vocab_size: int = 30522
    hidden_size: int = 768
    num_hidden_layers: int = 12
    num_attention_heads: int = 12
    intermediate_size: int = 3072
    hidden_act: str = "gelu"
    hidden_dropout_prob: float = 0.1
    attention_probs_dropout_prob: float = 0.1
    max_position_embeddings: int = 512
    type_vocab_size: int = 2
    initializer_range: float = 0.02
    layer_norm_eps: float = 1e-12
    pad_token_id: int | None = 0
    bos_token_id: int | None = None
    eos_token_id: int | list[int] | None = None
    embedding_size: int = 768
    q_groups: int = 4
    k_groups: int = 4
    v_groups: int = 4
    post_attention_groups: int = 1
    intermediate_groups: int = 4
    output_groups: int = 4
    tie_word_embeddings: bool = True


__all__ = ["SqueezeBertConfig"]
