#                🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨
#           This file was automatically generated from src/transformers/models/higgs_audio_v2/modular_higgs_audio_v2.py.
#               Do NOT edit this file manually as any edits will be overwritten by the generation of
#             the file from the modular. If any change should be done, please apply the change to the
#                          modular_higgs_audio_v2.py file directly. One of our CI enforces this.
#                🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨
# Copyright 2025 the HuggingFace Inc. team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from huggingface_hub.dataclasses import strict

from ...configuration_utils import PreTrainedConfig
from ...modeling_rope_utils import RopeParameters
from ...utils import auto_docstring
from ...utils.type_validators import interval


@auto_docstring(checkpoint="bosonai/higgs-audio-v2-generation-3B-base")
@strict
class HiggsAudioV2Config(PreTrainedConfig):
    r"""
    audio_bos_token_id (`int`, *optional*, defaults to 128013):
        The token ID for the beginning-of-sequence token for audio output.
    audio_delay_token_id (`int`, *optional*, defaults to 128014):
        The token ID used for audio delay pattern in multi-codebook generation.
    audio_stream_bos_id (`int`, *optional*, defaults to 1024):
        The ID for the beginning-of-stream token in audio sequences.
    audio_stream_eos_id (`int`, *optional*, defaults to 1025):
        The ID for the end-of-stream token in audio sequences.

    Example:

    ```python
    >>> from transformers import HiggsAudioV2Model, HiggsAudioV2Config

    >>> # Initializing a HiggsAudioV2 style configuration
    >>> configuration = HiggsAudioV2Config()

    >>> # Initializing a model from the configuration
    >>> model = HiggsAudioV2Model(configuration)

    >>> # Accessing the model configuration
    >>> configuration = model.config
    ```"""

    model_type = "higgs_audio_v2"
    keys_to_ignore_at_inference = ["past_key_values"]
    # Default tensor parallel plan for base model `HiggsAudioV2Model`
    base_model_tp_plan = {
        "layers.*.self_attn.q_proj": "colwise",
        "layers.*.self_attn.k_proj": "colwise",
        "layers.*.self_attn.v_proj": "colwise",
        "layers.*.self_attn.o_proj": "rowwise",
        "layers.*.mlp.gate_proj": "colwise",
        "layers.*.mlp.up_proj": "colwise",
        "layers.*.mlp.down_proj": "rowwise",
    }
    base_model_pp_plan = {
        "embed_tokens": (["input_ids"], ["inputs_embeds"]),
        "layers": (["hidden_states", "attention_mask"], ["hidden_states"]),
        "norm": (["hidden_states"], ["hidden_states"]),
    }

    vocab_size: int = 128256
    hidden_size: int = 3072
    intermediate_size: int = 8192
    num_hidden_layers: int = 28
    num_attention_heads: int = 24
    num_key_value_heads: int = 8
    hidden_act: str = "silu"
    max_position_embeddings: int = 2048
    initializer_range: float = interval(min=0.0, max=1.0)(default=0.02)
    rms_norm_eps: float = 1e-5
    use_cache: bool = True
    pad_token_id: int | None = 128001
    bos_token_id: int | None = 1
    eos_token_id: int | list[int] | None = 128009
    pretraining_tp: int | None = 1
    tie_word_embeddings: bool = False
    rope_parameters: RopeParameters | dict | None = None
    attention_bias: bool = False
    attention_dropout: int | float | None = 0.0
    mlp_bias: bool = False
    head_dim: int | None = 128
    num_codebooks: int = 8
    codebook_size: int = 1024
    audio_token_id: int = 128016
    audio_bos_token_id: int = 128013
    audio_delay_token_id: int = 128014
    audio_stream_bos_id: int = 1024
    audio_stream_eos_id: int = 1025

    def __post_init__(self, **kwargs):
        if self.rope_parameters is None:
            self.rope_parameters = {
                "factor": 32.0,
                "rope_theta": 500000.0,
                "high_freq_factor": 0.5,
                "low_freq_factor": 0.125,
                "original_max_position_embeddings": 1024,
                "rope_type": "llama3",
            }
        if self.head_dim is None:
            self.head_dim = self.hidden_size // self.num_attention_heads
        if self.num_key_value_heads is None:
            self.num_key_value_heads = self.num_attention_heads

        super().__post_init__(**kwargs)

    def validate_architecture(self):
        """Part of `@strict`-powered validation. Validates the architecture of the config."""
        if self.hidden_size % self.num_attention_heads != 0:
            raise ValueError(
                f"The hidden size ({self.hidden_size}) is not a multiple of the number of attention "
                f"heads ({self.num_attention_heads})."
            )


__all__ = ["HiggsAudioV2Config"]
